//! Provides helper functions that replace the standard `std.testing.expect*`
//! functions. The goal is to switch the type coersion such that `actual`
//! determines the type of `expected`, avoiding the use of `@as`.

const std = @import("std");
const meta = @import("../meta.zig");

pub fn @"true"(value: anytype) !void {
    return equal(true, value);
}

pub fn @"false"(value: anytype) !void {
    return equal(false, value);
}

pub fn @"null"(value: anytype) !void {
    return equal(null, value);
}

pub fn err(expected: anyerror, actual: anytype) !void {
    return std.testing.expectError(expected, actual);
}

pub fn equal(expected: anytype, actual: anytype) !void {
    return if (comptime meta.isZigString(@TypeOf(expected)))
        equalStrings(expected, actual)
    else
        std.testing.expectEqual(@as(@TypeOf(actual), expected), actual);
}

pub fn equalDeep(expected: anytype, actual: anytype) !void {
    return std.testing.expectEqualDeep(@as(@TypeOf(actual), expected), actual);
}

pub fn equalStrings(expected: []const u8, actual: anyerror!?[]const u8) !void {
    const actual_not_error = actual catch |e| {
        std.debug.print("expected string, found {}", .{e});
        return error.TestExpectedEqual;
    };
    if (actual_not_error) |actual_not_optional| {
        return std.testing.expectEqualStrings(expected, actual_not_optional);
    } else {
        std.debug.print("expected string, found null", .{});
        return error.TestExpectedEqual;
    }
}

pub fn fmt(expected: []const u8, comptime template: []const u8, args: anytype) !void {
    return std.testing.expectFmt(expected, template, args);
}
